﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EPiServer.Web;
using EPiServer.Core;
using System.Text.RegularExpressions;
using System.Collections.Specialized;
using log4net;
using System.Globalization;

namespace EPiServer.PageProviders
{
    /// <summary>
    /// Handles URL rewriting for a page provider
    /// </summary>
    public class PageProviderUrlModule : IHttpModule
    {
        #region IHttpModule Members

        private PageReference _entryPoint;
        private static object _setupLock = new object();

        private const string PageProviderName = "NW";
        private Regex _urlValidate = new Regex(@"\.(?<id>\d+)/$",RegexOptions.Compiled);

        private static ILog _log = LogManager.GetLogger(typeof(PageProviderUrlModule));


        public void Dispose()
        {
        }

        public void Init(HttpApplication context)
        {
            // This method will be called multiple times for every HttpApplication, since the UrlRewriteModule has one
            // instance per application with non-static events this is perfectly fine.
            UrlRewriteModule m = context.Modules["UrlRewriteModule"] as UrlRewriteModule;
            if(m==null)
                throw new Exception("Cannot find a http module with name UrlRewriteModule");

            m.HttpRewritingToInternal += new EventHandler<UrlRewriteEventArgs>(m_HttpRewritingToInternal);
        }

        private void m_HttpRewritingToInternal(object sender, UrlRewriteEventArgs e)
        {
            if (e.Cancel || e.Url==null)
            {
                return;
            }

            // Don't do anything when edit-mode passes id=xx since then no parsing is needed
            if (e.Url.QueryCollection["id"] != null)
            {
                return;
            }

            // Locking is handled in Init-method
            if(PageReference.IsNullOrEmpty(_entryPoint))
            {
                Init();
            }

            // Check if the path follows "our" pattern
            Match match = _urlValidate.Match(e.Url.Path);
            if (!match.Success)
            {
                return;
            }

            // Now verify that the path is rooted on our page provider,
            // we do not cache this path since the page could be moved
            string url = GetFriendlyURL(_entryPoint);
            if (e.Url.Path.ToString().StartsWith(url,true,CultureInfo.CurrentCulture))
            {
                PageReference pageLink;

                // Try to parse the "id" baked into the URL
                if (PageReference.TryParse(match.Groups["id"].Value, out pageLink))
                {
                    //Get the page from our page provider
                    PageData page = DataFactory.Instance.GetPage(new PageReference(pageLink.ID,PageProviderName));
                    
                    // Rewrite the path and keep any query strings
                    Url urlStaticLink = new Url(page.StaticLinkURL);
                    e.Url.Path = urlStaticLink.Path;
                    e.Url.QueryCollection.Add(urlStaticLink.QueryCollection);

                    // Let EPi know we did some changes
                    e.IsModified = true;
                    e.Internal = page.PageLink;
                    e.Cancel = true;
                }
            }
        }

        /// <summary>
        /// Helper method to resolve friendly URL for a page
        /// </summary>
        /// <param name="pageLink">The PageReference</param>
        /// <returns></returns>
        private string GetFriendlyURL(PageReference pageLink)
        {
            PageData page = DataFactory.Instance.GetPage(pageLink);
            UrlBuilder url = new UrlBuilder(EPiServer.UriSupport.AddLanguageSelection(page.LinkURL, page.LanguageBranch));
            EPiServer.Global.UrlRewriteProvider.ConvertToExternal(url, page.PageLink, System.Text.UTF8Encoding.UTF8);
            return url.Path;
        }

        /// <summary>
        /// Initialize our configs
        /// </summary>
        private void Init()
        {
            lock (_setupLock)
            {
                if (PageReference.IsNullOrEmpty(_entryPoint))
                {
                    PageProviderBase p = DataFactory.Instance.ProviderMap.GetPageProvider(PageProviderName);
                    if (p == null)
                        throw new Exception("Cannot find an instance of page provider " + PageProviderName);

                    _entryPoint = p.EntryPoint;
                }
            }
        }

        #endregion
    }
}
